@@ -131,6 +131,39 @@ class ShellTest(testtools.TestCase):
         self.assertEqual(_('Profile Type not found: wrong_type'),
                          six.text_type(ex))
 
+    @mock.patch.object(utils, 'format_output')
+    def test_do_profile_type_operations(self, mock_format):
+        service = mock.Mock()
+        fake_pto = {'foo': 'bar'}
+        service.list_profile_type_operations = mock.Mock(return_value=fake_pto)
+        args_dict = {
+            'format': 'json',
+            'type_name': 'os.nova.server-1.0'
+        }
+        args = self._make_args(args_dict)
+        sh.do_profile_type_ops(service, args)
+        mock_format.assert_called_with({'foo': 'bar'}, format=args.format)
+        service.list_profile_type_operations.assert_called_with(
+            'os.nova.server-1.0')
+        args.format = None
+        sh.do_profile_type_ops(service, args)
+        mock_format.assert_called_with({'foo': 'bar'})
+
+    def test_do_profile_type_operations_type_not_found(self):
+        service = mock.Mock()
+        args = {
+            'type_name': 'wrong_type',
+            'format': 'json'
+        }
+        args = self._make_args(args)
+        ex = oexc.ResourceNotFound
+        service.list_profile_type_operations = mock.Mock(side_effect=ex)
+        ex = self.assertRaises(exc.CommandError,
+                               sh.do_profile_type_ops,
+                               service, args)
+        self.assertEqual(_('Profile Type not found: wrong_type'),
+                         six.text_type(ex))
+
     @mock.patch.object(utils, 'print_list')
     def test_do_profile_list(self, mock_print):
         service = mock.Mock()
@@ -834,11 +867,19 @@ class ShellTest(testtools.TestCase):
 
     def test_do_cluster_delete(self):
         service = mock.Mock()
-        args = {'id': ['CID']}
+        args = {'id': ['CID'], 'force_delete': False}
         args = self._make_args(args)
         service.delete_cluster = mock.Mock()
         sh.do_cluster_delete(service, args)
-        service.delete_cluster.assert_called_once_with('CID', False)
+        service.delete_cluster.assert_called_once_with('CID', False, False)
+
+    def test_do_cluster_delete_force(self):
+        service = mock.Mock()
+        args = {'id': ['CID'], 'force_delete': True}
+        args = self._make_args(args)
+        service.delete_cluster = mock.Mock()
+        sh.do_cluster_delete(service, args)
+        service.delete_cluster.assert_called_once_with('CID', True, False)
 
     @mock.patch('subprocess.Popen')
     def test__run_script(self, mock_proc):
@@ -1561,6 +1602,76 @@ class ShellTest(testtools.TestCase):
         service.create_node.assert_called_once_with(**attrs)
         mock_show.assert_called_once_with(service, 'node_id')
 
+    @mock.patch.object(sh, '_show_node')
+    def test_do_node_adopt(self, mock_show):
+        args = {
+            'identity': 'fake-resoruce-id',
+            'name': 'adopt-node1',
+            'role': 'master',
+            'metadata': ['user=demo'],
+            'snapshot': None,
+            'overrides': '{"networks": [{"network": "fake-net-name"}]}',
+            'type': 'os.nova.server-1.0',
+            'preview': False
+        }
+        args = self._make_args(args)
+        attrs = {
+            'identity': 'fake-resoruce-id',
+            'name': 'adopt-node1',
+            'role': 'master',
+            'metadata': {'user': 'demo'},
+            'overrides': {'networks': [{'network': 'fake-net-name'}]},
+            'snapshot': None,
+            'type': 'os.nova.server-1.0',
+        }
+        service = mock.Mock()
+        node = mock.Mock()
+        node.id = 'node_id'
+        service.adopt_node.return_value = node
+        sh.do_node_adopt(service, args)
+        service.adopt_node.assert_called_once_with(**attrs)
+        mock_show.assert_called_once_with(service, 'node_id')
+
+    @mock.patch.object(utils, 'print_dict')
+    @mock.patch.object(utils, 'nested_dict_formatter')
+    def test_do_node_adopt_preview(self, mock_nest, mock_print):
+        args = {
+            'identity': 'fake-resoruce-id',
+            'snapshot': None,
+            'overrides': '{"networks": [{"network": "fake-net-name"}]}',
+            'type': 'os.nova.server-1.0',
+            'preview': True
+        }
+        args = self._make_args(args)
+        attrs = {
+            'identity': 'fake-resoruce-id',
+            'overrides': {'networks': [{'network': 'fake-net-name'}]},
+            'snapshot': None,
+            'type': 'os.nova.server-1.0',
+        }
+
+        fake_preview = {
+            "node_profile": {
+                "node_preview": {
+                    "properties": {
+                    },
+                    "type": "os.nova.server",
+                    "version": "1.0"}
+            }
+        }
+
+        service = mock.Mock()
+        service.adopt_node.return_value = fake_preview
+        sh.do_node_adopt(service, args)
+        service.adopt_node.assert_called_once_with(True, **attrs)
+
+        formatters = {}
+        formatters['node_preview'] = utils.nested_dict_formatter(
+            ['type', 'version', 'properties'],
+            ['property', 'value'])
+        mock_print.assert_called_once_with(fake_preview['node_profile'],
+                                           formatters=formatters)
+
     @mock.patch.object(sh, '_show_node')
     def test_do_node_show(self, mock_show):
         service = mock.Mock()
@@ -1599,12 +1710,21 @@ class ShellTest(testtools.TestCase):
 
     def test_do_node_delete(self):
         service = mock.Mock()
-        args = self._make_args({'id': ['node1']})
+        args = self._make_args({'id': ['node1'], 'force_delete': False})
         service.delete_node = mock.Mock()
 
         sh.do_node_delete(service, args)
 
-        service.delete_node.assert_called_once_with('node1', False)
+        service.delete_node.assert_called_once_with('node1', False, False)
+
+    def test_do_node_delete_force(self):
+        service = mock.Mock()
+        args = self._make_args({'id': ['node1'], 'force_delete': True})
+        service.delete_node = mock.Mock()
+
+        sh.do_node_delete(service, args)
+
+        service.delete_node.assert_called_once_with('node1', True, False)
 
     def test_do_node_check(self):
         service = mock.Mock()
