@@ -96,6 +96,28 @@ def do_profile_type_show(service, args):
         print(utils.format_output(pt))
 
 
+@utils.arg('type_name', metavar='<TYPE_NAME>',
+           help=_('Profile type to retrieve.'))
+@utils.arg('-F', '--format', metavar='<FORMAT>',
+           choices=utils.supported_formats.keys(),
+           help=_("The template output format, one of: %s.")
+                 % ', '.join(utils.supported_formats.keys()))
+def do_profile_type_ops(service, args):
+    """Show the operations about a policy type."""
+    show_deprecated('senlin profile-type-ops',
+                    'openstack cluster profile type ops')
+    try:
+        ops = service.list_profile_type_operations(args.type_name)
+    except sdk_exc.ResourceNotFound:
+        raise exc.CommandError(_('Profile Type not found: %s')
+                               % args.type_name)
+
+    if args.format:
+        print(utils.format_output(ops, format=args.format))
+    else:
+        print(utils.format_output(ops))
+
+
 # PROFILES
 
 @utils.arg('-f', '--filters', metavar='<"KEY1=VALUE1;KEY2=VALUE2...">',
@@ -631,6 +653,8 @@ def do_cluster_collect(service, args):
 
 @utils.arg('id', metavar='<CLUSTER>', nargs='+',
            help=_('Name or ID of cluster(s) to delete.'))
+@utils.arg('-f', '--force-delete', default=False, action="store_true",
+           help=_('Force to delete cluster(s).'))
 def do_cluster_delete(service, args):
     """Delete the cluster(s)."""
     show_deprecated('senlin cluster-delete', 'openstack cluster delete')
@@ -638,7 +662,7 @@ def do_cluster_delete(service, args):
     result = {}
     for cid in args.id:
         try:
-            cluster = service.delete_cluster(cid, False)
+            cluster = service.delete_cluster(cid, args.force_delete, False)
             result[cid] = ('OK', cluster.location.split('/')[-1])
         except Exception as ex:
             result[cid] = ('ERROR', six.text_type(ex))
@@ -1329,6 +1353,71 @@ def do_node_create(service, args):
     _show_node(service, node.id)
 
 
+@utils.arg('-i', '--identity', metavar='<IDENTITY>', required=True,
+           help=_('Physical resource id.'))
+@utils.arg('-t', '--type', metavar='<TYPE>', required=True,
+           help=_('The name of the profile type.'))
+@utils.arg('-r', '--role', metavar='<ROLE>',
+           help=_('Role for this node in the specific cluster.'))
+@utils.arg('-M', '--metadata', metavar='<"KEY1=VALUE1;KEY2=VALUE2...">',
+           help=_('Metadata values to be attached to the node. '
+                  'This can be specified multiple times, or once with '
+                  'key-value pairs separated by a semicolon.'),
+           action='append')
+@utils.arg('-n', '--name', metavar='<NAME>',
+           help=_('The name for the node.'))
+@utils.arg('-o', '--overrides', metavar='<JSON">',
+           help=_('JSON formatted specification for overriding this node '
+                  'properties.'))
+@utils.arg('-p', '--preview', default=False,
+           help=_('Whether preview the node adopt request. If set, '
+                  'only previewing this node and do not adopt.'),
+           action='store_true')
+@utils.arg('-s', '--snapshot', default=False,
+           help=_('Whether a shapshot of the existing physical object should '
+                  'be created before the object is adopted as a node.'),
+           action='store_true')
+def do_node_adopt(service, args):
+    """Adopt (or preview) a node."""
+    show_deprecated('senlin node-adopt', 'openstack cluster node adopt')
+    if args.preview:
+        _do_node_adopt_preview(service, args)
+    else:
+        _do_node_adopt(service, args)
+
+
+def _do_node_adopt_preview(service, args):
+    attrs = {
+        'identity': args.identity,
+        'overrides': utils.format_json_parameter(args.overrides),
+        'snapshot': args.snapshot,
+        'type': args.type
+    }
+
+    node = service.adopt_node(True, **attrs)
+
+    formatters = {}
+    formatters['node_preview'] = utils.nested_dict_formatter(
+        ['type', 'version', 'properties'],
+        ['property', 'value'])
+    utils.print_dict(node['node_profile'], formatters=formatters)
+
+
+def _do_node_adopt(service, args):
+    attrs = {
+        'identity': args.identity,
+        'name': args.name,
+        'role': args.role,
+        'metadata': utils.format_parameters(args.metadata),
+        'overrides': utils.format_json_parameter(args.overrides),
+        'snapshot': args.snapshot,
+        'type': args.type
+    }
+
+    node = service.adopt_node(**attrs)
+    _show_node(service, node.id)
+
+
 @utils.arg('-D', '--details', default=False, action="store_true",
            help=_('Include physical object details.'))
 @utils.arg('id', metavar='<NODE>',
@@ -1341,6 +1430,8 @@ def do_node_show(service, args):
 
 @utils.arg('id', metavar='<NODE>', nargs='+',
            help=_('Name or ID of node(s) to delete.'))
+@utils.arg('-f', '--force-delete', default=False, action="store_true",
+           help=_('Force to delete the node(s).'))
 def do_node_delete(service, args):
     """Delete the node(s)."""
     show_deprecated('senlin node-delete', 'openstack cluster node delete')
@@ -1348,7 +1439,7 @@ def do_node_delete(service, args):
     result = {}
     for nid in args.id:
         try:
-            node = service.delete_node(nid, False)
+            node = service.delete_node(nid, args.force_delete, False)
             result[nid] = ('OK', node.location.split('/')[-1])
         except Exception as ex:
             result[nid] = ('ERROR', six.text_type(ex))
